package Evo::Io::Handle;
use Evo '-Class::Out *';

with '-Io::Handle::Role';

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Evo::Io::Handle

=head1 VERSION

version 0.0209

=head1 AUTHOR

alexbyk.com

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by alexbyk.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
